"""
This module is for testing new functionalities
that could be implemented in future versions.

The working and tested functions are out of
this module.

Use these functions at your own risk.

TODO: Check the imports, refactor and make it
work in a near future or remove it if not
possible.
"""